function asve=ASVE(data, parameter)

%ASVE - Adaptive spot (co-)volatility estimator 
%Given a data vector of length n that contains high-frequency observation 
%of a semimartingale corrupted by additive noise, the function returns the 
%spot volatility estimator introduced in Sabel, Schmidt-Hieber, and Munk 
%(2013). 
%After a data cleaning procedure eliminating jumps in the time series, the
%function does pre-averaging followed by a wavelet decomposition. 
%Thresholding of the wavelet coefficients is done via SURE block 
%thresholding (cf. Cai and Zhou (2009)). Additionally, a rudimentary beta 
%version for spot covolatility estimation is given. Note that the 
%parameters in the procedure are optimized for spot volatility estimation 
%only.
%
% Syntax:  asve = ASVE(data,parameter)
%
% Inputs:
%   data        -   structure containing the obligatory field 
%
%                   data1:      [1xn double] first or only time series 
%                               to be analysed 
%
%                   to perform a covolatility estimation, one must define
%
%                   data2:      [1xn double] second time series, which 
%                               is synchronous to the first one
%    
%   parameter   -   optional, structure containing some of the following 
%                   fields:
%
%                   preav_fct:  [char] pre-average function, must be one of
%
%                               'cos'  - lambda1 in Table 1 in Sabel et al.
%                               'block'- lambda3 in Table 1 in Sabel et al.
%                               'sin'  - lambda4 in Table 1 in Sabel et al.
%                               
%                               default is 'sin'
%
%                   preav_c:    [double] preav_c*sqrt(n) is the number of 
%                               preaveraged values. By default, the 
%                               preav_c is chosen to be the theoretically 
%                               optimal constant in dependence of the 
%                               empirical signal-to-noise ratio.
%
%                   wavelet:    [char] determines the wavelet basis (see
%                               wfilters.m to find a list of
%                               possibilities). By default, Haar wavelets
%                               'db1' are chosen.
%                   naive_qv:   [logical] indicates whether microstructure
%                               noise should be ignored and volatility
%                               should be estimated via realized quadratic 
%                               variation (1) or if pre-averaging should be
%                               applied (0).
%
% Outputs:
%   asve        -   vector of length m containing the estimated spot
%                   (co-)volatility at the i/m*n-th time point
%
% Example: 
%   
%   clear data parameter
%   %simulate data:
%   n=30001; %sample size
%   sigma=  @(x) 1+(x>1/2)+(x<1/3)/2;  %take spot vol to be a jump function
%   semimartingale=zeros(1,10001);
%   for i=2:n
%       semimartingale(i)=semimartingale(i-1)+ ...
%                          sum(sigma((i-1)/n+(1:100)/100/n).* ...
%                          randn(1,100)*sqrt(1/n/100));
%   end %generates the contnuous semimartingale part
%   
%   % scaling is chosen similar to log-returns of FGBL-data
%   data.data1=semimartingale/300+0.0002*randn(1,n);  %these are the data
%
%   %with pre-averaging:
%   parameter.naive_qv=0;
%
%   asve=ASVE(data,parameter); %do spot volatility estimation
%
%   %plotting
%   m=length(asve);
%   plot((1:m)/m,asve,'LineWidth',2)
%   hold on
%   plot((0:100)/100,(sigma((0:100)/100)/300).^2,'r','LineWidth',2)
%   hold off
%
%   %naive approach without pre-averaging:
%   parameter.naive_qv=1;
%   asve_naive=ASVE(data,parameter);
%
% Other m-files required:   preaveraging.m, SURE.m, Wavelet Toolbox
% Subfunctions:             calc_l_0, wave_vec_normalize, compute_SNR, 
%                           jumptest, local_t_test, jumptestincr
% MAT-files required:       none
%
% See also: preaveraging.m, SURE.m
%
% Authors: Till Sabel and Johannes Schmidt-Hieber
% Georg-August-Universitaet Goettingen and ENSAE
% email: tsabel@uni-goettingen.de
% Website: http://www.stochastik.math.uni-goettingen.de/
% Nov 2013; Last revision: 16-Jan-2014



%------------- BEGIN CODE --------------

%%% BEGIN defining default values
if nargin<2
    parameter.tmp=0;
end

if isempty(intersect(fieldnames(data),'data1')) || ...
        prod(size(data.data1)-[1 0])>0
    error('No data input or wrong data format.')
end

if isempty(intersect(fieldnames(data),'data2'))
    data.data2=data.data1;
else
    if prod(size(data.data2)-[1 0])>0 || ...
            sum(abs(size(data.data1)-size(data.data2)))>0
        error('Sizes of first and second time series are not consistent.')
        data.data2=data.data1;
    else
        fprintf('Covolatility estimation is performed.')
    end
end

if isempty(intersect(fieldnames(parameter),'preav_fct'))
    parameter.preav_fct='sin';
end

if isempty(intersect(fieldnames(parameter),'preav_c'))
    SNR=computeSNR(data);
    switch parameter.preav_fct
        case 'cos'
            parameter.preav_c=SNR*0.49;
        case 'block'
            parameter.preav_c=SNR*0.35;
        case 'sin'
            parameter.preav_c=SNR*0.3;
    end
end

if isempty(intersect(fieldnames(parameter),'wavelet'))
    parameter.wavelet='db1';
end

if isempty(intersect(fieldnames(parameter),'naive_qv'))
    parameter.naive_qv=0;
end

if parameter.naive_qv
    warning(['Naive estimation via quadratic variation and ignoring '...
    'microstructure effects is performed'])
end
%%% END defining default values


n=length(data.data1)-1;
m=parameter.preav_c*sqrt(n);
m=n/floor(n/m);
parameter.m=m;

%%% pre-averagering:
% preav_data_vec are the pre-averaged values, 'sin' version is for
% approximation coefficients

if(parameter.naive_qv==0)
    parameter_sin=parameter;
    parameter_sin.preav_fct='sin';
    preav_data_vec_sin=preaveraging(data,parameter_sin);
    preav_data_vec=preaveraging(data,parameter);
else
    preav_data_vec_sin=n*(data.data1(2:end)-data.data1(1:(end-1))).* ...
        (data.data2(2:end)-data.data2(1:(end-1)));
    preav_data_vec=preav_data_vec_sin;
end


%%% Jump detection:
% detects jumps and replaces corresponding data from preav_data_vec by
% mean of its neighbors


m_jump=n^(3/4);
m_jump=n/floor(n/m_jump);
jumps=jumptest(data,'cos',m_jump)+n/m_jump;
jumps2=jumptestincr(data,1);
jumps=sort(unique([jumps2,transpose(jumps)]));

blocks_cont_jumps=[0]; %index of blocks containing a jump
for i=1:length(preav_data_vec)
    block_length=round(n*(i+1)/m)-round(n*(i-1)/m);
    if intersect(round(n*(i-1)/m)+ 1+(0:block_length),jumps)
        blocks_cont_jumps=[blocks_cont_jumps i];
    end
end

blocks_cont_jumps=blocks_cont_jumps(2:end);

if length(blocks_cont_jumps)==length(preav_data_vec)
    warning(['There seems to be a jump in the support of each ' ...
        'pre-average value. This might be the case if the signal is ' ...
        'too small compared to the noise level and the number of '...
        'observations. All values are replaced by integrated volatility.'])
    IV=mean(preav_data_vec);
    for j=1:length(preav_data_vec)
        preav_data_vec(j)=IV;
    end
else
    compl_blocksjumps=setdiff(1:(length(preav_data_vec)-1), ...
        blocks_cont_jumps);
    
    for j=blocks_cont_jumps
        neighbors=compl_blocksjumps(find(min(abs(j-compl_blocksjumps)) ...
            ==abs(j-compl_blocksjumps)));
        preav_data_vec(j)=mean(preav_data_vec(neighbors));
        preav_data_vec_sin(j)=mean(preav_data_vec_sin(neighbors));
    end
end

% to avoid boundary problems:
preav_data_vec=preav_data_vec(1:(end-1));
preav_data_vec_sin=preav_data_vec_sin(1:(end-1));

%%% Characteristics of the wavelet basis:
% If wave_length denotes the length of the wavelet filter, then
% decomposition is done in the following way:
% The lowest level contain approximation and first order detail 
% coefficients of length 2*wave_length - 1. The recursion for the length of
% the levels above is then a_(r+1)=2*a_r-2*wave_length+2. So, we determine 
% the smallest a_r larger than the original length of our data vector. The 
% corresponding r is stored in l_0 (highest resolution level).

wave_length=length(wfilters(parameter.wavelet))/2;
l_0=calc_l_0(length(preav_data_vec), wave_length);



%%% Wavelet decomposition

% for approximation coefficients (no thresholding afterwards)
[C_sin, L_sin]=wavedec(preav_data_vec_sin,l_0,parameter.wavelet);
% for detail coefficients
[C, L]=wavedec(preav_data_vec,l_0,parameter.wavelet);

% automatic thresholding of highest resolution levels
q=0.3;
k=ceil(q*l_0);
if(k>1)
    sum_L=sum(L((end-k+1):(end-1)));
else
    sum_L=0;
end
C=[C(1:(end-sum_L)) zeros(1,sum_L)];


%%% SURE Block Thresholding
% Homoscedacity of all wavelet coefficients is necessary. Therefore, the
% coefficients are normalized via wave_vec_normalize.
[C_norm std_vec]=wave_vec_normalize(preav_data_vec, C, L, wave_length);

% SURE block of the normalized coeff. (see SURE.m for details)

% First lev levels are not thresholded but replaced by corresponding
% empirical coefficients of sine-pre-averaged data.
lev=3;
C_SURE=SURE(C_norm,L, lev);

% inverting the normalization
C_SURE=C_SURE.*std_vec;
C_SURE(1:sum(L(1:lev)))=C_sin(1:sum(L(1:lev)));


%%% Wavelet Reconstruction
asve=waverec(C_SURE,L,parameter.wavelet);


% volatility estimation restricted to sigma^2>=0 (due to bias correction,
% negative values are possible)

if sum(data.data1==data.data2)==n+1
    asve=(asve>0).*asve;
end
return




% calc_l_0 computes the N for the wavedec routine. We choose N such that
% the wavelet decomposition returns all wavelet coefficients starting with
% the smallest possible resolution level. For  Haar wavelets ('db1')
% N is the smallest integer not smaller than log_2(length_of_signal).

function l_0=calc_l_0(len, wave_length)

do_while=1;
true_length=2*wave_length-1;
l_0=0;
while(do_while==1)
    true_length=2*true_length-2*wave_length+2;
    l_0=l_0+1;
    if(true_length>=len)
        do_while=0;
    end
end
return




% wave_vec_normalize normalizes wavelet coefficients by dividing by the
% standard deviation of the data vector in the respective interval.

function [C std_vec]=wave_vec_normalize(preav_data_vec, C, L, wave_length)

m=length(preav_data_vec);

% K= maximal level for which variance is computed
K=4;

std_vec=zeros(1,sum(L(1:(length(L)-1))));

wave_support_length=2*wave_length-1;
% Length of wavelet support is length(wavelet filter)-1 and wave_length
% is wavelet filter/2

for i=2:(length(L)-1)
    for r=1:L(i)
        if((i-2)<=K)
            
            I=max(1,floor(m*(r-wave_support_length)/L(i))):min(m, ...
                ceil(m*r/L(i)));
        else
            %determine the midpoint of the support of \psi_{j,k}
            mid_point=mean([max(0,(r-wave_support_length)/L(i)) min(1, ...
                r/L(i))]);
            
            %keep away from boundaries
            mid_point_adjusted=min(max(mid_point, wave_support_length/ ...
                L(K+3)), 1-wave_support_length/L(K+3));
            
            if(wave_support_length>=L(K+3)*2)
                I=[1:m];
            else
                I=max(floor(m*(mid_point_adjusted - wave_support_length/...
                    L(K+3))),1):min(ceil(m*(mid_point_adjusted + ...
                    wave_support_length/L(K+3))),m);
            end
            
        end
        
        std_vec(sum(L(1:(i-1)))+r)=std(preav_data_vec(I));
        
        C(sum(L(1:(i-1)))+r)=C(sum(L(1:(i-1)))+r)/ ...
            (std_vec(sum(L(1:(i-1)))+r)+eps);
    end
end
return

% computeSNR.m computes the estimator of the signal to noise ratio (SNR), 
% i.e. the quotient (<1,sigma^2>/<1,tau^2>)^(1/2). It is used for 
% determining the optimal block length for the preaveraging step in ASVE.m.
% <1,sigma^2> is estimated via preaveraging with m=sqrt(n), while the noise
% level is estimated via quadratic variation.


function SNR=computeSNR(data)

n=length(data.data1)-1;
m=sqrt(length(data.data1));
m=n/floor(n/m);
parameterSNR.m=m;
parameterSNR.preav_fct='sin';
preav_data_vec=preaveraging(data,parameterSNR);
noiselevel=sum((data.data1(2:end)-data.data1(1:(end-1))).^2)/ (4*n) + ...
    sum((data.data2(2:end)-data.data2(1:(end-1))).^2)/(4*n);
IV=sum(abs(preav_data_vec))/m;
SNR=sqrt(max(IV,0)/noiselevel);

return

% This test is designed to find small jumps in the data, which would
% disturb the estimation precedure immensely. It is based on a local
% t-test. More details may be found in Sabel, Schmidt-Hieber, Munk (2013),
% p.16 f


function jumps=jumptest(data,lambdastr,m)

n=length(data.data1)-1;

switch lambdastr
    case 'block'
        lambda= @(x) sqrt(3/2)*((x<1 & x>=0) - (x<=2 & x>1));         
    case 'sin'
        lambda = @(x) pi/sqrt(3)*sin(x*pi);
    case 'cos'
        lambda= @(x) pi/2*cos(x*pi/2); 
end

m=n/floor(n/m);

lambda_vec=lambda(1-m/n*(0:(n/m-1)));

pre_details_jump_1=zeros(1,n-2*length(lambda_vec)+2);
pre_details_jump_2=zeros(1,n-2*length(lambda_vec)+2);

for i=(length(lambda_vec)+1):(n-length(lambda_vec)+1)
    pre_details_jump_1(i-length(lambda_vec))=n/m*sum(lambda_vec.* ...
        (data.data1(i+(1:length(lambda_vec))) - ...
        data.data1(i-(1:length(lambda_vec)))));
    pre_details_jump_2(i-length(lambda_vec))=n/m*sum(lambda_vec.* ...
        (data.data2(i+(1:length(lambda_vec))) - ...
        data.data2(i-(1:length(lambda_vec)))));    
end

jumps_1=local_t_test(pre_details_jump_1,2*length(lambda_vec)+1);
jumps_2=local_t_test(pre_details_jump_2,2*length(lambda_vec)+1);

jumps=find((jumps_1==1)+(jumps_2==1)>0);

return



function detected_jumps=local_t_test(x,length_preav)

detected_jumps=zeros(length(x),1);
for block=1:ceil(length(x)/(length_preav.^2))
    currentx=x(((block-1)*length_preav.^2+1):min(length(x), ...
        block*length_preav.^2));
    est_sigma_vec=zeros(length_preav,1);
    for j=1:length_preav
        index_subsampling=(0:(length_preav-1))*length_preav+j;
        index_subsampling=index_subsampling(index_subsampling<= ...
            length(currentx));
        est_sigma_vec(j)=std(currentx(index_subsampling));
    end
    est_sigma=mean(est_sigma_vec);
    detected_jumps(((block-1)*length_preav.^2+1):min(length(x), ...
        block*length_preav.^2))=abs(currentx-mean(currentx))/ ...
        est_sigma>2.81;
end

return

% This jump test is based on extrem value arguments to find intervals where
% are huge consecutive jumps in the data. Further details can be found in
% Sabel, Schmidt-Hieber, Munk (2013), p.17f

function jumps=jumptestincr(data,c)

n=length(data.data1)-1;

qv_1=sum((data.data1(2:end)-data.data1(1:(end-1))).^2)/n;
jumps_1=(data.data1(2:end)-data.data1(1:(end-1))).^2<2*log(n)*qv_1*c;

qv_2=sum((data.data2(2:end)-data.data2(1:(end-1))).^2)/n;
jumps_2=(data.data2(2:end)-data.data2(1:(end-1))).^2<2*log(n)*qv_2*c;

jumps=1-jumps_1.*jumps_2;
temp=1:n;
jumps=temp(logical(jumps));

return

%------------- END OF CODE --------------
