
function C_SURE=SURE(C_norm,L, lev)  

%SURE - SURE block thresholding
%For normalized (i.e. std=1) wavelet coefficients C_norm, C_SURE computes
%the SURE block thresholded wavelet coeffiecients introduced in Cai and
%Zhou (2009).
%
% Syntax:  C_SURE = SURE(C_norm,L,lev)
%
% Inputs:
%   C_norm      -   vector of wavelet coefficients obtained by the function
%                   wavedec.m in the Wavelet Toolbox;
%                   Note that in heteroscedastic models, those coefficients
%                   have to be normalized, s.t. the data in the support of
%                   a coefficient has standard deviation 1.
%   
%   L           -   vector of information about level sizes obtained by 
%                   wavedec.m, look there for more details
%
%   lev         -   level up to which no thresholding should be performed,
%                   i.e. the lev-th level is the highest approximation
%                   level (optinal, default: lev=3)
%   
% Outputs:
%   C_SURE      -   vector of thresholded wavelet coeffients
%
% Example: 
%   
%   signal=[ones(1,512) 2*ones(1,512)];
%   std=0.1;
%   noisy_signal=signal+(randn(1,1024).^2-1)/sqrt(2)*std;
% 
%   [C L]=wavedec(noisy_signal,10,'db1');
%
%   C_SURE=SURE(C/std,L,3);
%   
%   reconstruction=waverec(C_SURE*std,L,'db1');
%
% Other m-files required:   Wavelet Toolbox            
% Subfunctions:             none
% MAT-files required:       none
%
% See also: ASVE.m, wavedec.m, waverec.m

% Authors: Till Sabel and Johannes Schmidt-Hieber
% Georg-August-Universitaet Goettingen and ENSAE
% email: tsabel@uni-goettingen.de
% Website: http://www.stochastik.math.uni-goettingen.de/
% Nov 2013; Last revision: 21-Nov-2013



%------------- BEGIN CODE --------------

if nargin<3
    lev=3;
end

C_SURE=C_norm;

for i= lev:(length(L)-2)
      
    levelI=C_norm(sum(L(1:i))+1:sum(L(1:(i+1))));
    d=length(levelI);
    
    T_d=1/d*sum(levelI.^2-1);
    gamma_d=d^(-1/2)*log2(d)^(3/2);
    
    if(T_d<=gamma_d)
        curr=1-2*log(d)./(levelI.^2+eps);
        C_SURE(sum(L(1:i))+1:sum(L(1:(i+1))))=curr.*(curr>0).*levelI;
    else
        sure_lambda=Inf*ones(floor(sqrt(d)), d+1);
        
        %j is block length, denote by L in the paper
        for j=1:floor(sqrt(d))            
            lambdaF=2*log(d)*j;
            if (j==1)
                sb_vec=[0 levelI.^2];
            else
                s=zeros(1,ceil(d/j));
                for b=1:ceil(d/j)
                    blockb=levelI( ((b-1)*j+1):(min(j*b,d)));
                    
                    s(b)=norm(blockb);
                    
                end
                Is=find((s.^2)>=j-2);
                sb_vec=[j-2 s(Is).^2];
            end
            IsF=find(sb_vec<=lambdaF);
            sb_vec=sb_vec(IsF);
            
            counter=0;
            for lambda=sb_vec
                counter=counter+1;
                sure_temp=0;
                for b=1:ceil(d/j)
                    blockb=levelI( ((b-1)*j+1):(min(j*b,d)));
                    
                    sb=norm(blockb)+eps;
                    
                    sure_temp=sure_temp+j+(lambda^2-2*lambda*(j-2))/ ...
                        sb^2*(sb^2>lambda)+(sb^2-2*j)*(sb^2<=lambda);
                end
                sure_lambda(j,counter)=sure_temp;
            end
        end
        % location of the minimum
        [jstar_temp counterstar_temp]=find(sure_lambda<= ...
            min(min(sure_lambda)));
        jstar=jstar_temp(1);
        counterstar=counterstar_temp(1);
        lambdaF=2*log(d)*jstar;
        
        s=zeros(1,ceil(d/jstar));
        
        for b=1:ceil(d/jstar)
            s(b)=norm(levelI( ((b-1)*jstar+1):(min(jstar*b,d))))+eps;
        end
        
        
        if (jstar==1)
            sb_vec=[0 levelI.^2];
           
        else
            Is=find((s.^2)>=jstar-2);
            sb_vec=[jstar-2 s(Is).^2];
        end
        IsF=find(sb_vec<=lambdaF);
        sb_vec=sb_vec(IsF);
        
        lambdastar=sb_vec(counterstar);
        
        for b=1:ceil(d/jstar)
            curr=1-lambdastar/s(b)^2; 
            C_SURE(sum(L(1:i))+(((b-1)*jstar+1):(min(jstar*b,d))))=curr*...
                (curr>0)*levelI(((b-1)*jstar+1):(min(jstar*b,d)));
        end
    end    
end  

return
%------------- END OF CODE --------------
