function preav_data_vec=preaveraging(data, parameter)

%preaveraging - Pre-averaging of data corrupted by additional noise  
%For data of length n from a given semimartingale corrupted by additional 
%noise, the function gives the pre-averaged values defined in Sabel, 
%Schmidt-Hierber, Munk (2013).
%
% Syntax:  preav_data_vec=preaveraging(data,parameter)
%
% Inputs:
%   data            -   structure containing the obligatory field 
%
%                   data1:      [1xn double] first or only time series 
%                               to be analysed 
%
%                   data2:      [1xn double] second time series, which 
%                               is synchronous to the first one;
%                               in order to perform volatility estimation,
%                               choose data2=data1.
%    
%   parameter       -   structure containing the obligatory fields:
%
%                   preav_fct:  [char] pre-average function, must be one of
%
%                               'cos'  - lambda1 in Table 1 in Sabel et al.
%                               'block'- lambda3 in Table 1 in Sabel et al.
%                               'sin'  - lambda4 in Table 1 in Sabel et al.
%                               
%                               default is 'sin'
%
%                   m:          [integer] number of preaveraged values
%
% Outputs:
%   preav_data_vec  -   vector of length m containing the pre-averaged
%                       values for spot (co-)volatility at the i/m*n-th 
%                       time point
%
% Example: 
%   
%   clear data parameter
%   %simulate data:
%   n=30001;
%   sigma=  @(x) 1+(x>1/2)+(x<1/3)/2;
%   semimartingale=zeros(1,10001);
%   for i=2:n
%       semimartingale(i)=semimartingale(i-1)+ ...
%                          sum(sigma((i-1)/n+(1:100)/100/n).* ...
%                          randn(1,100)*sqrt(1/n/100));
%   end
%   
%   % scaling is chosen similar to log-returns of FGBL-data
%   data.data1=semimartingale/300+0.0002*randn(1,n);  
%   data.data2=data.data1;
%
%   parameter.preav_fct='sin';
%   parameter.m=1428;
%
%   preav_data_vec=preaveraging(data, parameter);
%
%
% Other m-files required:   none
% Subfunctions:             none
% MAT-files required:       none
%
% See also: ASVE.m

% Authors: Till Sabel and Johannes Schmidt-Hieber
% Georg-August-Universitaet Goettingen and ENSAE
% email: tsabel@uni-goettingen.de
% Website: http://www.stochastik.math.uni-goettingen.de/
% Nov 2013; Last revision: 16-Jan-2014



%------------- BEGIN CODE --------------
n=length(data.data1)-1;
m=parameter.m;

switch parameter.preav_fct
    case 'block'
        lambda= @(x) sqrt(3/2)*((x<1 & x>=0) - (x<=2 & x>1));
        
    case 'sin'
        lambda = @(x) pi/sqrt(3)*sin(x*pi);
        
    case 'cos'
        lambda= @(x) pi/2*cos(x*pi/2);
end

pre_av_vec=zeros(1,floor(m-1));
bias_corr=zeros(1,floor(m-1));

% In comparison to the paper, entries are shifted one to the left due to 
% appended 0. i-th pre-average block consists of n(i-2)/m < j <= n i/m.
% Since j is at position j+1, the constraint reads n(i-2)/m < j-1 <= n i/m.
% which can be rewritten as floor(n*(i-2)/m)+1<=j-1<= floor(n*i/m).

% increment vector
diff_vec=(data.data1(2:(n+1))-data.data1(1:n)).* ...
    (data.data2(2:(n+1))-data.data2(1:n));

temp_lambda=lambda(1-m/n*(0:(n/m-1)));

% In comparison to the paper, entries are shifted one to the left due to 
% appended 0. The i-th pre-average block originally consists of j s.t. 
% n(i-2)/m < j <= n i/m. % Since j is at position j+1, the constraint reads
% n(i-2)/m < j-1 <= n i/m which can be rewritten as 
% floor(n*(i-2)/m)+1<=j-1<= floor(n*i/m).

for i=1:(m-1)
    pre_av_vec_1(i)=m/n*sum(temp_lambda.* ...
        (data.data1(round(i/m*n+(1:(n/m))+1))- ...
        data.data1(round(i/m*n-(1:(n/m))+1))));
    pre_av_vec_2(i)=m/n*sum(temp_lambda.* ...
        (data.data2(round(i/m*n+(1:(n/m))+1))- ...
        data.data2(round(i/m*n-(1:(n/m))+1))));
    bias_corr(i)=m^2/(2*n^2)*sum((temp_lambda.^2).* ...
        (diff_vec(round(i/m*n+(1:(n/m))))+ ...
        diff_vec(round(i/m*n-(1:(n/m))+1))));
end

% rounding is not necessary in theory, however, matlab sometimes returns
% k.0000000 instead of k

% normalization: Due to discretization effects, we have to normalize 
% lambda, so that it is a pre-average function.

normvec1=zeros(1,round(n/m));
normvec2=zeros(1,round(n/m));
for i1=1:(n/m)
    for i2=1:(n/m)
        normvec2(i2)=2*temp_lambda(i2)*temp_lambda(i1)*min(i1,i2);
    end
    normvec1(i1)=sum(normvec2)-temp_lambda(i1)^2;
end
norm=m^2/n^3*sum(normvec1);

if sum(data.data1==data.data2)==length(data.data1)
    preav_data_vec=[pre_av_vec_1.^2-bias_corr 0]/norm;
else
    preav_data_vec=[pre_av_vec_1.*pre_av_vec_2 0]/norm;
end

return
