function GUI()

%GUI - Graphical User Interface for Spot Volatility Estimation
%This function is a graphical interface for the function ASVE.m. It will
%ask, whether the data come from a file or instead simulated data should be
%used.
% -If the simulation option is chosen, the latent price process is 
%  generated by the Heston model with parameters 
%  rho=-2/3, theta=10^(-5), kappa=4, epsilon=sqrt(kappa*theta).
%  Here, the Euler Scheme is used for simulations (cf. Andersen (2007), 
%  available atSSRN 946405)
% -If the read-in option is selected, the function GUI will ask for a file 
%  name. This file should contain two columns (separated by a tab) with 
%  trading times and recorded prices, respectively.
% 
% Syntax:   GUI()
%
% Inputs:
%   none
%
% Outputs:
% 	The routine returns two graphs. The upper displays the price process. 
%   The lower gives the estimated spot volatility (together with the true 
%   spot volatility if simulated data are used).
%
% Other m-files required:   ASVE.m, preaveraging.m, SURE.m, Wavelet Toolbox
% Subfunctions:             simulate_data
% MAT-files required:       none
% txt-files required:       data_example.txt
%
% See also: ASVE.m, preaveraging.m, SURE.m

% Authors: Till Sabel and Johannes Schmidt-Hieber
% Georg-August-Universitaet Goettingen and ENSAE
% email: tsabel@uni-goettingen.de
% Website: http://www.stochastik.math.uni-goettingen.de/
% Jan 2014; Last revision: 16-Jan-2014



%------------- BEGIN CODE --------------

addpath('../Code')

datatype=listdlg('ListString', {'simulate data' 'read data from file'}, ...
  'SelectionMode', 'Single', 'Name', 'Simulate data?','ListSize',[160,33]);


switch datatype
    case 1
        n=inputdlg({'n'},'Data size?',1,{'15000'});
        n=eval(cell2mat(n));
           
        data=simulate_data(n);
        asve=ASVE(data);
        time=(1:length(asve))/length(asve);
    case 2

        [filename,pathname]=uigetfile('*.txt', 'Select data file', ...
            'data_example.txt');
        
        fid=fopen([pathname filename],'r');
        readdata=textscan(fid,'%f %f');
        fclose(fid);
        
        price=[readdata{2}]';
        data.data1=log(price)-log(price(1));
        time=[readdata{1}]';
        asve=ASVE(data);
end

%%% Plotting:

%rescaling figure:
hold off
handle_fig=figure(1);
close(handle_fig)
handle_fig=figure(1);
position=get(handle_fig,'Position');
position(3)=position(3)+position(1);
position(1)=position(1)/2;
set(handle_fig,'Position',position);


subplot(2,1,1)
if datatype==1
    plot((1:n)/n,exp(data.data1(2:end))*(100+rand(1,1)))
else
    plot(time,price)
end
title('Asset price')


subplot(2,1,2)
if datatype==1
    plot((1:n)/n,data.sigma.^2,'r','LineWidth',2)
    hold on
    plot(time,asve,'b','LineWidth',2)
    legend('truth','reconstruction',0)
    hold off
else
    rate=length(time)/length(asve);
    plot(time(round((rate/2):rate:end)),asve,'b','LineWidth',2)
end
title('Reconstruction of spot volatility')

return



%The next function generates a data example following the Heston model.

function data=simulate_data(n)

n32=n*floor(n^(1/2)); %n*[n^{1/2}]\approx [n^{3/2}]
n12=floor(n^(1/2));

automatic=1; %determines if heston parameters are automatically set
if automatic==0
    options.Resize='on';
    input_heston=inputdlg({'rho','theta','kappa','epsilon'},['Paramete' ...
        'rs for Heston model'],1,{'-2/3','10^(-5)','4','2*10^(-2.5)'}, ...
        options);
else
    input_heston={'-2/3','10^(-5)','4','2*10^(-2.5)'};
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Parameters are rho, theta, kappa, epsilon.
    % If these are changed, SNR might differ significantly.
    % Therefore, in order to model financial data
    % the noise level should be adjusted accordingly.
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    fprintf(['Semi-martingale is simulated according to Heston ' ...
        'model with\n rho=-2/3,\n theta=10^(-5),\n kappa=4,\n ' ...
        'epsilon=sqrt(kappa*theta).\n'])
end

rho=eval(cell2mat(input_heston(1)));
theta=eval(cell2mat(input_heston(2)));
kappa=eval(cell2mat(input_heston(3)));
epsilon=eval(cell2mat(input_heston(4)));


%Generate Heston  model via Euler scheme 
%cf. Andersen (2007): Efficient simulation of the Heston stochastic 
%volatility model, available at SSRN id 946405

BM=zeros(1,n32);
V=zeros(1,n32);
Zv=randn(1,n32);
Zx=rho*Zv+sqrt(1-rho^2)*randn(1,n32);

BM(1)=0;
V(1)=theta;

delta=1/n32;

for i=1:(n32-1)
    tempV=max(V(i),0);       %since P(V<0)>0
    
    BM(i+1)=BM(i)-0.5*tempV*delta+sqrt(tempV)*Zx(i)*sqrt(delta);
    V(i+1)=V(i)+kappa*(theta-tempV)*delta+epsilon*sqrt(tempV)*Zv(i)* ... 
        sqrt(delta);
end

sigma=sqrt(max(0,V));
%semi-martingale part:
I=BM(n12:n12:end); 

%noise part:
tau=ones(1,n)/5000;
noise=tau.*randn(1,n);

data.data1=[0 I]+[0 noise];
data.brownianpart=I;
data.tau=tau;
data.sigma=sigma(n12:n12:n32);
data.time=[1:n]/n;
return

%------------- END OF CODE --------------
